import dayjs from 'dayjs';
import React, {Component} from 'react';
import {
  ActivityIndicator,
  Alert,
  Dimensions,
  ImageBackground,
  Platform,
  ScrollView,
  StatusBar,
  Text,
  TouchableOpacity,
  View,
  Image,
} from 'react-native';
import {KeyboardAwareScrollView} from 'react-native-keyboard-aware-scroll-view';
import MaterialIcons from 'react-native-vector-icons/MaterialIcons';
import {connect} from 'react-redux';
import * as colors from '../../assets/colors';
import img_popup from '../../assets/images/img_popup.png';
import ic_clock from '../../assets/images/ic_clock.png';
import ic_true from '../../assets/images/ic_true.png';
import ic_false from '../../assets/images/ic_false.png';
import SharePreference from '../../common/SharePreference';
import Config from '../../Config';
import {
  REQUEST_IS_FINISH,
  REQUEST_COMPONENTS_KEY_PAGES,
  REQUEST_CLEAR_DATA,
} from '../../redux/actions';
import {request} from '../../redux/api';
import CountDown from '../count-down';
import Button from './button';
import {styles} from './styles';
// Detect screen width and height
const {width, height} = Dimensions.get('window');
MaterialIcons.loadFont();
import SoundManager from '../../components/SoundManager';
import {hideCotrolAndroid} from '../../common/helper';
import QuestionIntro from './intro-question';

class OnboardingScreens extends Component {
  // Props for ScrollView component
  static defaultProps = {
    // Arrange screens horizontally
    horizontal: true,
    // Scroll exactly to the next screen, instead of continous scrolling
    pagingEnabled: true,
    // Hide all scroll indicators
    showsHorizontalScrollIndicator: false,
    showsVerticalScrollIndicator: false,
    // Do not bounce when the end is reached
    bounces: false,
    // Do not scroll to top when the status bar is tapped
    scrollsToTop: false,
    // Remove offscreen child views
    removeClippedSubviews: true,
    // Do not adjust content behind nav-, tab- or toolbars automatically
    automaticallyAdjustContentInsets: false,
    // Fisrt is screen is active
    index: 0,
  };

  componentDidMount = () => {
    hideCotrolAndroid();
  };

  state = this.initState(this.props);

  /**
   * Initialize the state
   */
  initState(props) {
    // Get the total number of slides passed as children
    const total = props.children ? props.children.length || 1 : 0,
      // Current index
      index = total > 1 ? Math.min(props.index, total - 1) : 0,
      // Current offset
      offset = width * index;

    const state = {
      total,
      index,
      offset,
      width,
      height,
      loading: false,
      tutorial: false,
    };

    // Component internals as a class property,
    // and not state to avoid component re-renders when updated
    this.internals = {
      isScrolling: false,
      offset,
    };
    this.oldIndex = 0;

    return state;
  }

  /**
   * Scroll begin handler
   * @param {object} e native event
   */
  onScrollBegin = (e) => {
    // Update internal isScrolling state
    this.internals.isScrolling = true;
  };

  /**
   * Scroll end handler
   * @param {object} e native event
   */
  onScrollEnd = (e) => {
    // Update internal isScrolling state
    this.internals.isScrolling = false;

    // Update index
    this.updateIndex(
      e.nativeEvent.contentOffset
        ? e.nativeEvent.contentOffset.x
        : // When scrolled with .scrollTo() on Android there is no contentOffset
          e.nativeEvent.position * this.state.width,
    );
  };

  /*
   * Drag end handler
   * @param {object} e native event
   */
  onScrollEndDrag = (e) => {
    const {
        contentOffset: {x: newOffset},
      } = e.nativeEvent,
      {children} = this.props,
      {index} = this.state,
      {offset} = this.internals;
    // Update internal isScrolling state
    // if swiped right on the last slide
    // or left on the first one
    if (
      offset === newOffset &&
      (index === 0 || index === children.length - 1)
    ) {
      this.internals.isScrolling = false;
    }
  };

  /**
   * Update index after scroll
   * @param {object} offset content offset
   */
  updateIndex = (offset) => {
    this.props.stopSound();
    const {dispatch} = this.props;
    const state = this.state,
      diff = offset - this.internals.offset,
      step = state.width;
    let index = state.index;

    // Do nothing if offset didn't change
    if (!diff) {
      return;
    }

    // Make sure index is always an integer
    index = parseInt(index + Math.round(diff / step), 10);

    // Update internal offset
    this.internals.offset = offset;
    // Update index in the state
    this.setState({index});
    dispatch({type: REQUEST_COMPONENTS_KEY_PAGES, payload: {keyPage: index}});
  };

  /**
   * Swipe one slide forward
   */
  swipe = () => {
    // Ignore if already scrolling or if there is less than 2 slides
    if (this.internals.isScrolling || this.state.total < 2) {
      return;
    }
    const state = this.state,
      diff = this.state.index + 1,
      x = diff * state.width,
      y = 0;
    // Call scrollTo on scrollView component to perform the swipe
    this.scrollView && this.scrollView.scrollTo({x, y, animated: true});

    // Update internal scroll state
    this.internals.isScrolling = true;

    // Trigger onScrollEnd manually on android
    if (Platform.OS === 'android') {
      setImmediate(() => {
        this.onScrollEnd({
          nativeEvent: {
            position: diff,
          },
        });
      });
    }
  };

  prev = () => {
    const {isRecording} = this.props;

    if (this.internals.isScrolling || this.state.total < 2) {
      return;
    }
    if (!isRecording) {
      const state = this.state,
        diff = this.state.index - 1,
        x = diff * state.width,
        y = 0;
      // Call scrollTo on scrollView component to perform the swipe
      this.scrollView && this.scrollView.scrollTo({x, y, animated: true});

      // Update internal scroll state
      this.internals.isScrolling = true;

      // Trigger onScrollEnd manually on android
      if (Platform.OS === 'android') {
        setImmediate(() => {
          this.onScrollEnd({
            nativeEvent: {
              position: diff,
            },
          });
        });
      }
    } else {
      this.notifyStopRecord();
    }
  };
  /**
   * Render ScrollView component
   * @param {array} slides to swipe through
   */
  renderScrollView = (pages) => {
    return (
      <ScrollView
        ref={(component) => {
          this.scrollView = component;
        }}
        {...this.props}
        style={{flex: 1}}
        onScrollBeginDrag={this.onScrollBegin}
        onMomentumScrollEnd={this.onScrollEnd}
        onScrollEndDrag={this.onScrollEndDrag}>
        {pages.map((page, i) => (
          // Render each slide inside a View
          <KeyboardAwareScrollView key={i}>
            <View style={[styles.slide]}>{page}</View>
          </KeyboardAwareScrollView>
        ))}
      </ScrollView>
    );
  };

  /**
   * Render pagination indicators
   */
  renderPagination = () => {
    if (this.state.total <= 1) {
      return null;
    }

    const ActiveDot = <View style={[styles.dot, styles.activeDot]} />,
      Dot = <View style={styles.dot} />;

    let dots = [];

    for (let key = 0; key < this.state.total; key++) {
      dots.push(
        key === this.state.index
          ? // Active dot
            React.cloneElement(ActiveDot, {key})
          : // Other dots
            React.cloneElement(Dot, {key}),
      );
    }

    return (
      <View pointerEvents="none" style={[styles.pagination, styles.fullScreen]}>
        {dots}
      </View>
    );
  };
  renderQuestions = () => {
    const last = this.state.index + 1;
    const total = this.state.total;
    this.percentQuestion();
    return (
      <View style={[styles.question]}>
        <Text style={styles.questionText}>
          {last < 10 ? '0' + last : last}/{total < 10 ? '0' + total : total}
        </Text>
        {/* <View style={styles.line} /> */}
      </View>
    );
  };
  /**
   * Render Continue or Done button
   */
  renderButton = () => {
    const {index, total} = this.state;
    const lastScreen = index === total - 1;

    return (
      <View pointerEvents="box-none" style={[styles.buttonWrapper]}>
        {!lastScreen && (
          <Button
            text="Câu tiếp"
            bgColor={colors.color_blue_2}
            onPress={() => this.onNext()}
          />
        )}
      </View>
    );
  };

  onNext = () => {
    const {questions, isRecording} = this.props;
    const {index} = this.state;
    // console.log(305, index, questions[index], questions[index + 1]);
    if (!isRecording) {
      const itemCurrent = questions[index];
      const itemNext = questions[index + 1];
      if (itemCurrent.section_tag === itemNext.section_tag) {
        this.swipe();
      } else {
        this.setState({tutorial: true});
        this.timeSkip = setTimeout(() => {
          this.onSkip();
        }, 3000);
      }
    } else {
      this.notifyStopRecord();
    }
  };

  notifyStopRecord = () => {
    Alert.alert(
      'Thông báo',
      'Bạn đang ở chế độ ghi phát âm thanh.\nBạn hãy tạm dừng để chuyển câu hỏi',
      [{text: 'Ok', onPress: () => hideCotrolAndroid()}],
    );
  };

  onSkip = () => {
    this.setState({tutorial: false});
    this.swipe();
    if (this.timeSkip) {
      clearTimeout(this.timeSkip);
    }
  };

  numberAnswer = () => {
    const {user_answer} = this.props.submitData;
    const isAnswer = user_answer.filter((obj) => obj.user_answer.value !== '');
    return isAnswer.length + '/' + user_answer.length;
  };

  confirmAnwer = (type) => {
    const {isFinishQiz, isResult, stopSound} = this.props;
    if (isFinishQiz || isResult) {
      return;
    }
    stopSound();
    if (type === 0) {
      Alert.alert(
        'Thông báo',
        `Bạn đã trả lời ${this.numberAnswer()} câu hỏi.\nHoàn thành bài kiểm tra?`,
        [{text: 'Huỷ bỏ'}, {text: 'Đồng ý', onPress: () => this.acceptAnwer()}],
      );
    } else {
      Alert.alert(
        'Thông báo',
        `Bạn đã hết thời gian làm bài.\nBạn đã hoàn thành ${this.numberAnswer()} câu hỏi.`,
      );
      setTimeout(() => {
        this.acceptAnwer();
      }, 2000);
    }
  };

  acceptAnwer = () => {
    const {data, submitData} = this.props;

    let dataReview = [];
    submitData.user_answer.forEach((item) => {
      const dataObj = {
        question_configuration: item.question_configuration,
        question_id: item.question_id,
        section_id: item.section_id,
        section_tag: item.section_tag,
        user_answer: item.user_answer,
        type: item.type,
      };
      dataReview.push(dataObj);
    });
    // dispath action request submit answer
    const dataAnswer = {
      data: JSON.stringify(dataReview),
      started_at: SharePreference.startTest,
      finished_at: dayjs().unix(),
    };
    if (SharePreference.isTest) {
      dataAnswer.user_test_id = data.data.user_test_id + '';
      this.postSubmitAnswer(dataAnswer);
    } else {
      this.callApiJoin(dataAnswer, data.data.id);
    }
  };

  callApiJoin = (params, id) => {
    const param = {test_id: id};
    // console.log(382, id);
    this.setState({loading: true});
    request({
      url: `${Config.url}/api/test/join`,
      method: 'post',
      headers: {Authorization: SharePreference.token_hotqa},
      body: param,
    })
      .then((res) => {
        if (res.body.status === 200 && res.body.success) {
          const {data} = res.body;
          params.user_test_id = data.id;
          // console.log(393, params);
          this.postSubmitAnswer(params);
        } else {
          this.dialogNotify(res.body.message, params, id);
          this.setState({loading: false});
        }
      })
      .catch((e) => {
        const error = JSON.parse(e.message);
        this.dialogNotify(error.message, params, id);
        this.setState({loading: false});
      });
  };

  postSubmitAnswer = (params) => {
    const {dispatch, navigation} = this.props;
    // console.log(418, params);

    this.setState({loading: true});
    request({
      url: `${Config.url}/api/test/submit-answers`,
      method: 'post',
      headers: {Authorization: SharePreference.token_hotqa},
      body: params,
    })
      .then((res) => {
        if (res.body.status === 200 && res.body.success) {
          dispatch({type: REQUEST_IS_FINISH, payload: {isFinish: true}});
          const {data} = res.body;
          // console.log('postSubmitAnswer', data);
          if (data) {
            const point = Math.round(
              (data.total_correct / data.max_score) * 10,
            );
            navigation.navigate('Congratulations', {
              numberAnswer: `${point}/10`,
            });
          } else {
            navigation.navigate('Congratulations', {numberAnswer: '?/10'});
          }
        } else {
          this.dialogNotify(res.body.message, params, null);
        }
        this.setState({loading: false});
      })
      .catch((e) => {
        // console.log(447, e);
        const error = JSON.parse(e.message);
        this.dialogNotify(error.message, params, null);
        this.setState({loading: false});
      });
  };

  dialogNotify = (message, params, id) => {
    const {route} = this.props;
    Alert.alert('Lỗi', message, [
      {text: 'Huỷ bỏ', onPress: () => (route ? route.goBack() : {})},
      {
        text: 'Thử lại',
        onPress: () => {
          if (id) {
            this.callApiJoin(params, id);
          } else {
            this.postSubmitAnswer(params);
          }
        },
      },
    ]);
  };

  renderButtonPrev = () => {
    const lastScreen = this.state.index >= 1;
    return (
      <View pointerEvents="box-none" style={[styles.buttonPrev]}>
        {lastScreen && (
          <Button
            text="Quay lại"
            bgColor={colors.color_blue_3}
            onPress={() => this.prev()}
          />
        )}
      </View>
    );
  };

  percentQuestion = () => {
    const last = this.state.index + 1;
    const total = this.state.total;
    const popupPercent = last * (95 / total);
    const progressPercent = last * (100 / total);
    // console.log(455, percent, popupPercent, progressPercent);
    return {popup: popupPercent, progress: progressPercent};
  };

  btnComplete = () => {
    const {isFinishQiz, navigation, isResult, data} = this.props;

    if (isFinishQiz || isResult) {
      let point;
      if (isResult) {
        const {info} = data;
        const minScore = Math.round((info.total_correct * 10) / info.max_score);
        point = minScore + '/10';
      } else {
        point = this.props.point;
      }
      navigation.replace('Congratulations', {numberAnswer: point});
      // navigation.reset({
      //   routes: [{name: 'Congratulations', params: {numberAnswer: point}}],
      // });
    } else {
      Alert.alert(
        'Thông báo',
        `Bạn đã trả lời ${this.numberAnswer()} câu hỏi. Bạn muốn hoàn thành bài kiểm tra?`,
        [{text: 'Huỷ bỏ'}, {text: 'Đồng ý', onPress: () => this.acceptAnwer()}],
      );
    }
  };

  isCorrect = () => {
    const {questions, keyPage} = this.props;
    if (questions[keyPage].is_correct === 1) {
      return true;
    }
    return false;
  };

  /**
   * Render the component
   */
  render = (
    {children, data, questions, keyPage, route, isResult, dispatch} = this
      .props,
  ) => {
    const {data: cur_data} = data;
    const {index} = this.state;

    return (
      <View style={[styles.container, styles.fullScreen]}>
        <StatusBar hidden={true} />

        <View style={styles.viewHeader}>
          <TouchableOpacity
            style={styles.btnBack}
            onPress={() => {
              if (route) {
                dispatch({type: REQUEST_CLEAR_DATA});
                SoundManager.destroy();
                route.goBack();
              }
            }}>
            <MaterialIcons name="keyboard-backspace" style={styles.iconBack} />
          </TouchableOpacity>
          <Text style={styles.txtTitle}>{questions[keyPage].titleSection}</Text>

          {isResult && (
            <View
              style={[
                styles.btnResult,
                {backgroundColor: this.isCorrect() ? '#44C149' : 'red'},
              ]}>
              <Image
                source={this.isCorrect() ? ic_true : ic_false}
                style={styles.img_result}
                resizeMode="contain"
              />
              <Text style={styles.txtResult}>
                {this.isCorrect() ? 'Đúng' : 'Sai'}
              </Text>
            </View>
          )}

          <View style={styles.btnTime}>
            <Image
              source={ic_clock}
              resizeMode="contain"
              style={styles.iconClock}
            />
            <CountDown
              minutes={cur_data.time_limit}
              endTime={() => this.confirmAnwer(1)}
            />
          </View>
          <TouchableOpacity style={styles.btnDone} onPress={this.btnComplete}>
            <Text style={styles.txtDone}>Hoàn thành</Text>
          </TouchableOpacity>
        </View>

        {this.renderScrollView(children)}
        <View style={styles.viewBottom}>
          <View style={styles.viewLeft}>
            <ImageBackground
              source={img_popup}
              resizeMode="contain"
              style={[
                styles.popup,
                {marginStart: this.percentQuestion().popup + '%'},
              ]}>
              {this.renderQuestions()}
            </ImageBackground>
            <View style={styles.viewProgress}>
              <View
                style={[
                  styles.progress,
                  {width: this.percentQuestion().progress + '%'},
                ]}
              />
            </View>
          </View>
          <View style={styles.viewRight}>
            {this.renderButtonPrev()}
            {this.renderButton()}
          </View>
        </View>
        {this.state.loading && (
          <ActivityIndicator
            size="large"
            color="orange"
            style={styles.loading}
          />
        )}
        {this.state.tutorial && (
          <QuestionIntro question={questions[index + 1]} onSkip={this.onSkip} />
        )}
      </View>
    );
  };
}
function mapStateToProps(state) {
  return {
    data: state.reducersAccepted,
    isFinishQiz: state.reducerComponents.isFinish,
    isResult: state.reducerComponents.isResult,
    point: state.reducerComponents.point,
    submitData: state.submitData,
    submitAnswer: state.submitAnswer,
  };
}
export default connect(mapStateToProps)(OnboardingScreens);
